'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

exports.instrument = instrument;
exports.report = report;
exports.thresholds = thresholds;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var coverageVariable = '__start_istanbul__';

function instrument() {
    var istanbul = arguments.length <= 0 || arguments[0] === undefined ? require('istanbul') : arguments[0];
    var options = arguments[1];

    return function (input) {
        return function istanbulInstrument(log) {
            return new _promise2.default(function (resolve, reject) {
                var instrumenter = new istanbul.Instrumenter((0, _extends3.default)({
                    embedSource: true,
                    noAutoWrap: true
                }, options, {
                    coverageVariable: coverageVariable
                }));

                istanbul.hook.hookRequire(
                // hook requires which matches input files
                function (file) {
                    return input.indexOf(file) !== -1;
                },
                // and instrument that sources
                function (source, file) {
                    var result = instrumenter.instrumentSync(source, file);

                    log(file);

                    return result;
                });

                resolve(input);
            });
        };
    };
}

function report() {
    var reporters = arguments.length <= 0 || arguments[0] === undefined ? ['lcovonly', 'text-summary'] : arguments[0];
    var dir = arguments.length <= 1 || arguments[1] === undefined ? 'coverage/' : arguments[1];

    return function (input) {
        return function istanbulReport() /* log */{
            var istanbul = require('istanbul');

            istanbul.hook.unhookRequire();

            return new _promise2.default(function (resolve, reject) {
                if (!global[coverageVariable]) {
                    return reject('no coverage information was collected');
                }

                var collector = new istanbul.Collector();
                var reporter = new istanbul.Reporter(null, dir);

                collector.add(global[coverageVariable]);
                reporter.addAll(reporters);
                reporter.write(collector, false /* async */, function () {
                    resolve(input);
                });
            });
        };
    };
}

function thresholds() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    return function (input) {
        return function istanbulThresholds() /* log */{
            var istanbul = require('istanbul');

            return new _promise2.default(function (resolve, reject) {
                var coverage = istanbul.utils.summarizeCoverage(global[coverageVariable]);
                var result = (0, _keys2.default)(options).reduce(function (errors, key) {
                    var threshold = options[key];
                    var summary = coverage[key];

                    // check percentage threshold
                    if (threshold > 0) {
                        if (summary.pct < threshold) {
                            return errors.concat(key + ' percentage: ' + summary.pct + '% < ' + threshold + '%');
                        }
                    }

                    // check gap threshold
                    if (threshold < 0) {
                        if (summary.covered - summary.total < threshold) {
                            return errors.concat(key + ' gap: ' + summary.covered + ' - ' + summary.total + ' < ' + threshold);
                        }
                    }

                    return errors;
                }, []);

                if (result.length > 0) {
                    return reject(result);
                }

                resolve(input);
            });
        };
    };
}